"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = checkField;

var _predicate = require("predicate");

var _predicate2 = _interopRequireDefault(_predicate);

var _utils = require("./utils");

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var POSITIVE_PREDICATE = _predicate2.default;
var NEGATIVE_PREDICATE = _predicate2.default.not;

function checkField(fieldVal, rule) {
  var predicator = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _predicate2.default;

  if (Array.isArray(fieldVal)) {
    // Simple rule - like emptyString
    return fieldVal.some(function (val) {
      return checkField(val, rule, predicator);
    });
  } else if ((0, _utils.isObject)(rule)) {
    // Complicated rule - like { greater then 10 }
    return Object.keys(rule).every(function (p) {
      var comparable = rule[p];
      if ((0, _utils.isObject)(comparable) || p === _constants.NOT) {
        if (p === _constants.OR || p === _constants.AND) {
          if (Array.isArray(comparable)) {
            if (p === _constants.OR) {
              return comparable.some(function (rule) {
                return checkField(fieldVal, rule, predicator);
              });
            } else {
              return comparable.every(function (rule) {
                return checkField(fieldVal, rule, predicator);
              });
            }
          } else {
            return false;
          }
        } else if (p === _constants.NOT) {
          var oppositePredicator = predicator === NEGATIVE_PREDICATE ? POSITIVE_PREDICATE : NEGATIVE_PREDICATE;
          return checkField(fieldVal, comparable, oppositePredicator);
        } else {
          return false;
        }
      } else {
        return predicator[p](fieldVal, comparable);
      }
    });
  } else {
    return predicator[rule](fieldVal);
  }
}